function [p_scat,p_ins]=scatsphere(r,th,a,k,N)
%[p_scat,p_inc]=SCATCYL(r,th,a,k,N)
%
%Computes the total sound field when a rigid sphere is insonified by a
%plane wave propagating along the x-axis
%
%r and theta are vectors or matrices of r and theta values
%a is the radius of the sphere
%k is the wavenumber (k=2*pi*f/c)
%N is the number of elements in the sum (default: 20)
%
%p_scat is a r by th matrix of the complex scattered pressures
%p_inc is a r by th matrix of the complex incidence pressures
%
%To create and plot a scattered soundfield specified in carthesian 
%coordinates:
%[X,Y]=meshgrid(linspace(minx,maxx,xres),linspace(miny,maxy,yres));
%[th,r]=cart2pol(X,Y);
%p_tot=scatsphere(r,th,a,k,N);
%surf(X,Y,real(p_s));
%
%To create and plot a scattered soundfield specified in polar 
%coordinates:
%th=linspace(0,2*pi,angle_res);
%r=linspace(a,maxr,rres);
%[p_s,p_i]=scatsphere(r,th,a,k);	
%[th_grid,r_grid]=meshgrid(th,r);
%[x,y]=pol2cart(th_grid,r_grid);
%surf(x,y,real(p_s));

%Per Waagoe, 2010

if nargin<5 %Default to order 20
    N=20;
end

if isvector(r) && isvector(th)
    %Both r and th are vectors. This means that the output matrix
    %is organized in polar coordinates (r by theta)
    pol=true; 
   
    if size(r,1)<size(r,2) %Make sure that r is a column vector
        r=r';
    end
    if size(th,2)<size(th,1) %Make sure that th is a row vector
        th=th';
    end
    
    %Compute incidence field
    p_ins=exp(1i*k*r*cos(th));
else
    %r and theta are matrices. The output matrix is organized in
    %carthesian coordinates
    
    pol=false;
    if size(r)~=size(th)
        error('Sizes of r and theta matrices mismatch');
    end
    
    %Compute incidence field
    p_ins=exp(1i*k*r.*cos(th));
end

p_scat=zeros(size(r,1),size(th,2));

%Define spherical bessel and hankel functions and their derivatives
j     = @(n,x) sqrt(pi./(2*x)).*besselj(n+0.5,x);
h     = @(n,x) sqrt(pi./(2*x)).*besselh(n+0.5,x);
jdiff = @(n,x) j(n-1,x) - j(n,x)*(n+1)./x;
hdiff = @(n,x) h(n-1,x) - h(n,x)*(n+1)./x;

%This loop computes N elements of the infinite sum in eq 3.23
for n=0:N
    pn_scat=-1i^n * (2*n+1) *jdiff(n,k*a).*h(n,k*r)./hdiff(n,k*a);
    
    if pol %Polar coordinates. Use matrix multiplication
        pn_scat=pn_scat*P(n,cos(th));
    else   %Carthesian coordinates. Use element by element multiplication 
        pn_scat=pn_scat.*P(n,cos(th));
    end
    
    p_scat=p_scat+pn_scat;
end

%Computations are not valid inside sphere
if ~pol
    p_ins(r<a)=NaN;
    p_scat(r<a)=NaN; 
end


